package com.dataflux.architect.example;

import com.dataflux.architect.*;
import java.math.BigDecimal;
import java.util.HashMap;

/**
 * Demonstrate {@link com.dataflux.architect.ISimpleStep} semantics.
 * <p>
 * This is an example implementation of
 * {@link com.dataflux.architect.ISimpleStep}, through extension of the
 * {@link com.dataflux.architect.SimpleStepAdapter}, that adds a value
 * to an input field and places the result in the new field; <code>sample_added</code>.
 * </p>
 * <p>
 * This example has two parameters.  The parameters are used to identify the field in which
 * to add and the value to add to the field value.
 * </p>
 * <p>
 * <strong>Note:</strong> The plugin examples are included in the Data Management Platform Java&#x2122;
 * Plugin jar and as such have a package name that represents that location.  Therefore, modifying the
 * examples requires placing the modified source, including the package statement, in a new Java&#x2122;
 * compilation unit.
 * </p>
 *
 * @author <a href="http://www.sas.com">SAS Institute</a>
 * @since dfPower Studio 8.0
 */
public class SimpleStepExample
       extends SimpleStepAdapter
{
   /** Name of the field to add. */
   private String mFieldToAdd;

   /** DataType of the field to add. */
   private DataType mFieldToAddType;

   /** Value to add to the field. */
   private BigDecimal mValueToAdd;

   /**
    * Parameter associated with the name of the input field that holds the value to add.
    */
   static final String FIELD_TO_ADD = "Field for arithmetic operation";

   /**
    * Name of the new sum field.
    */
   static final String SUM_FIELD = "sample_added";

   /**
    * Parameter associated with the value to add to the input field.
    */
   static final String VALUE_TO_ADD = "Value to add to the field";

   /**
    * Constructor.
    * <p>
    * This example requires two parameters.  To enable parameter processing by the
    * {@link com.dataflux.architect.SimpleStepAdapter} the constructor must
    * create/populate {@link com.dataflux.architect.SimpleStepAdapter#mParameterNames mParameterNames}
    * and {@link com.dataflux.architect.SimpleStepAdapter#mParameterValues mParameterValues}.
    * </p>
    */
   public SimpleStepExample()
   {
      mParameterValues   = new HashMap<String, String>();
      mParameterNames    = new String[2];
      mParameterNames[0] = FIELD_TO_ADD;
      mParameterNames[1] = VALUE_TO_ADD;
   }

   public void processRow()
          throws StepException
   {
      Object value = mValues.get(mFieldToAdd);

      // It's possible the source value is null, if so, we cannot add to it
      if (null == value)
         {
         mValues.put(SUM_FIELD, value);
         return;
         }

      // Add to the source value
      if (DataType.REAL == mFieldToAddType)
         mValues.put(SUM_FIELD, ((BigDecimal)value).add(mValueToAdd));
      else
         {
         double valueAsDouble = ((Integer)value).intValue();
         BigDecimal result = mValueToAdd.add(new BigDecimal(valueAsDouble));
         mValues.put(SUM_FIELD, result);
         }
   }

   public void validateInputs()
          throws StepException
   {
      // Create our output fields needs
      mOutputFields = new FieldInformation[mInputFields.length + 1];

      /**
       * Validate the parameters.
       *
       * Start with verifying the value to add is a number.
       */
      try
         {
         mValueToAdd = new BigDecimal(mParameterValues.get(VALUE_TO_ADD));
         }
      catch (NullPointerException e)
         {
         throw new StepException("The '"
                                 + VALUE_TO_ADD
                                 + "' parameter must be specified.");
         }
      catch (NumberFormatException e)
         {
         throw new StepException("The '"
                                 + VALUE_TO_ADD
                                 + "' parameters must be a number.");
         }

      // Verify the name of the field to add.
      if (null == mParameterValues.get(FIELD_TO_ADD))
         {
         throw new StepException("The '"
                                 + FIELD_TO_ADD
                                 + "' parameter must be specified.");
         }
      for (int i = 0; i < mInputFields.length; i++)
         {
         // This example returns all input fields
         mOutputFields[i] = mInputFields[i];

         // If we found the field, remember it
         if (mInputFields[i].getName().equals(mParameterValues.get(FIELD_TO_ADD)))
            {
            // Make sure the specified field is a numeric
            if ((DataType.INTEGER != mInputFields[i].getType()) && (DataType.REAL != mInputFields[i].getType()))
               throw new StepException("Incorrect field type. Must be integer or real.");

            // Remember the field
            mFieldToAdd     = mInputFields[i].getName();
            mFieldToAddType = mInputFields[i].getType();
            }
         }
      if (null == mFieldToAdd)
         {
         throw new StepException("Could not find field named '"
                                 + mParameterValues.get(FIELD_TO_ADD)
                                 + "' in input.");
         }

      // Add our sum field to the set of output fields
      mOutputFields[mInputFields.length] = new FieldInformation(SUM_FIELD, DataType.REAL, 0);
   }
}
